@@ -5,6 +5,10 @@
  *  This module implements the default usage procedure for
  *  Automated Options.  It may be overridden, of course.
  *
+ * @addtogroup autoopts
+ * @{
+ */
+/*
  *  Sort options:
     --start=END-[S]TATIC-FORWARD --patt='^/\*($|[^:])' \
     --out=xx.c key='^[a-zA-Z0-9_]+\(' --trail='^/\*:' \
@@ -14,7 +18,7 @@
 /*
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -26,38 +30,74 @@
  *   The Modified Berkeley Software Distribution License
  *      See the file "COPYING.mbsd"
  *
- *  These files have the following md5sums:
+ *  These files have the following sha256 sums:
  *
- *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
- *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
- *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
+ *  8584710e9b04216a394078dc156b781d0b47e1729104d666658aecef8ee32e95  COPYING.gplv3
+ *  4379e7444a0e2ce2b12dd6f5a52a27a4d02d39d247901d3285c88cf0d37f477b  COPYING.lgplv3
+ *  13aa749a5b0a454917a944ed8fffc530b784f5ead522b1aacaf4ec8aa55a6239  COPYING.mbsd
  */
 
 #define OPTPROC_L_N_S  (OPTPROC_LONGOPT | OPTPROC_SHORTOPT)
+#if defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
+# include <libintl.h>
+#endif
+
+typedef struct {
+    size_t          fnm_len;
+    uint32_t        fnm_mask;
+    char const *    fnm_name;
+} ao_flag_names_t;
+
+/**
+ * Automated Options Usage Flags.
+ * NB: no entry may be a prefix of another entry
+ */
+#define AOFLAG_TABLE                            \
+    _aof_(gnu,             OPTPROC_GNUUSAGE )   \
+    _aof_(autoopts,        ~OPTPROC_GNUUSAGE)   \
+    _aof_(no_misuse_usage, OPTPROC_MISUSE   )   \
+    _aof_(misuse_usage,    ~OPTPROC_MISUSE  )   \
+    _aof_(compute,         OPTPROC_COMPUTE  )
+
+#define _aof_(_n, _f)   AOUF_ ## _n ## _ID,
+typedef enum { AOFLAG_TABLE AOUF_COUNT } ao_flag_id_t;
+#undef  _aof_
+
+#define _aof_(_n, _f)   AOUF_ ## _n = (1 << AOUF_ ## _n ## _ID),
+typedef enum { AOFLAG_TABLE } ao_flags_t;
+#undef  _aof_
 
 /* = = = START-STATIC-FORWARD = = = */
+static unsigned int
+parse_usage_flags(ao_flag_names_t const * fnt, char const * txt);
+
 static inline bool
 do_gnu_usage(tOptions * pOpts);
 
 static inline bool
 skip_misuse_usage(tOptions * pOpts);
 
+static void
+print_offer_usage(tOptions * opts);
+
 static void
 print_usage_details(tOptions * opts, int exit_code);
 
 static void
-prt_conflicts(tOptions * pOptions, tOptDesc * pOD);
+print_one_paragraph(char const * text, bool plain, FILE * fp);
 
 static void
-prt_one_vendor(tOptions * pOptions, tOptDesc * pOD,
-               arg_types_t * pAT, char const * usefmt);
+prt_conflicts(tOptions * opts, tOptDesc * od);
 
 static void
-prt_vendor_opts(tOptions * pOpts, char const * pOptTitle);
+prt_one_vendor(tOptions *    opts,  tOptDesc *   od,
+               arg_types_t * argtp, char const * usefmt);
 
 static void
-prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
-               char const * pOptTitle);
+prt_vendor_opts(tOptions * opts, char const * title);
+
+static void
+prt_extd_usage(tOptions * opts, tOptDesc * od, char const * title);
 
 static void
 prt_ini_list(char const * const * papz, bool * need_intro,
@@ -82,94 +122,125 @@ static int
 setStdOptFmts(tOptions * opts, char const ** ptxt);
 /* = = = END-STATIC-FORWARD = = = */
 
-/*
- *  NB: no entry may be a prefix of another entry
+/**
+ * Parse the option usage flags string.  Any parsing problems yield
+ * a zero (no flags set) result.  This function is internal to
+ * set_usage_flags().
+ *
+ * @param[in] fnt   Flag Name Table - maps a name to a mask
+ * @param[in] txt   the text to process.  If NULL, then
+ *                  getenv("AUTOOPTS_USAGE") is used.
+ * @returns a bit mask indicating which \a fnt entries were found.
  */
-#define AOFLAG_TABLE                            \
-    _aof_(gnu,             OPTPROC_GNUUSAGE )   \
-    _aof_(autoopts,        ~OPTPROC_GNUUSAGE)   \
-    _aof_(no_misuse_usage, OPTPROC_MISUSE   )   \
-    _aof_(misuse_usage,    ~OPTPROC_MISUSE  )
-
-LOCAL void
-set_usage_flags(tOptions * opts, char const * flg_txt)
+static unsigned int
+parse_usage_flags(ao_flag_names_t const * fnt, char const * txt)
 {
-    typedef struct {
-        size_t          fnm_len;
-        uint32_t        fnm_mask;
-        char const *    fnm_name;
-    } ao_flag_names_t;
+    unsigned int res = 0;
 
-#   define _aof_(_n, _f)   AOUF_ ## _n ## _ID,
-    typedef enum { AOFLAG_TABLE AOUF_COUNT } ao_flag_id_t;
-#   undef  _aof_
-
-#   define _aof_(_n, _f)   AOUF_ ## _n = (1 << AOUF_ ## _n ## _ID),
-    typedef enum { AOFLAG_TABLE } ao_flags_t;
-#   undef  _aof_
-
-#   define _aof_(_n, _f)   { sizeof(#_n)-1, _f, #_n },
-    static ao_flag_names_t const fn_table[AOUF_COUNT] = {
-        AOFLAG_TABLE
-    };
-#   undef  _aof_
-
-    unsigned int flg = (ao_flags_t)0;
-
-    if (flg_txt == NULL) {
-        flg_txt = getenv("AUTOOPTS_USAGE");
-        if (flg_txt == NULL) return;
+    /*
+     * The text may be passed in.  If not, use the environment variable.
+     */
+    if (txt == NULL) {
+        txt = getenv("AUTOOPTS_USAGE");
+        if (txt == NULL)
+            return 0;
     }
 
-    flg_txt = SPN_WHITESPACE_CHARS(flg_txt);
-    if (*flg_txt == NUL)
-        return;
+    txt = SPN_WHITESPACE_CHARS(txt);
+    if (*txt == NUL)
+        return 0;
 
+    /*
+     * search the string for table entries.  We must understand everything
+     * we see in the string, or we give up on it.
+     */
     for (;;) {
         int ix = 0;
-        ao_flag_names_t const * fnt = fn_table;
 
         for (;;) {
-            if (strneqvcmp(flg_txt, fnt->fnm_name, (int)fnt->fnm_len) == 0)
+            if (strneqvcmp(txt, fnt[ix].fnm_name, (int)fnt[ix].fnm_len) == 0)
                 break;
             if (++ix >= AOUF_COUNT)
-                return;
-            fnt++;
+                return 0;
         }
 
         /*
          *  Make sure we have a full match.  Look for whitespace,
          *  a comma, or a NUL byte.
          */
-        if (! IS_END_LIST_ENTRY_CHAR(flg_txt[fnt->fnm_len]))
-            return;
+        if (! IS_END_LIST_ENTRY_CHAR(txt[fnt[ix].fnm_len]))
+            return 0;
 
-        flg |= 1U << ix;
-        flg_txt = SPN_WHITESPACE_CHARS(flg_txt + fnt->fnm_len);
+        res |= 1U << ix;
+        txt = SPN_WHITESPACE_CHARS(txt + fnt[ix].fnm_len);
 
-        if (*flg_txt == NUL)
-            break;
+        switch (*txt) {
+        case NUL:
+            return res;
 
-        if (*flg_txt == ',') {
-            /*
-             *  skip the comma and following white space
-             */
-            flg_txt = SPN_WHITESPACE_CHARS(flg_txt + 1);
-            if (*flg_txt == NUL)
-                break;
+        case ',':
+            txt = SPN_WHITESPACE_CHARS(txt + 1);
+            /* Something must follow the comma */
+
+        default:
+            continue;
         }
     }
+}
 
+/**
+ * Set option usage flags.  Any parsing problems yield no changes to options.
+ * Three different bits may be fiddled: \a OPTPROC_GNUUSAGE, \a OPTPROC_MISUSE
+ * and \a OPTPROC_COMPUTE.
+ *
+ * @param[in] flg_txt   text to parse.  If NULL, then the AUTOOPTS_USAGE
+ *                      environment variable is parsed.
+ * @param[in,out] opts  the program option descriptor
+ */
+LOCAL void
+set_usage_flags(tOptions * opts, char const * flg_txt)
+{
+#   define _aof_(_n, _f)   { sizeof(#_n)-1, _f, #_n },
+    static ao_flag_names_t const fn_table[AOUF_COUNT] = {
+        AOFLAG_TABLE
+    };
+#   undef  _aof_
+
+    /*
+     * the flag word holds a bit for each selected table entry.
+     */
+    unsigned int flg = parse_usage_flags(fn_table, flg_txt);
+    if (flg == 0) return;
+
+    /*
+     * Ensure we do not have conflicting selections
+     */
+    {
+        static unsigned int const form_mask =
+            AOUF_gnu | AOUF_autoopts;
+        static unsigned int const misuse_mask =
+            AOUF_no_misuse_usage | AOUF_misuse_usage;
+        if (  ((flg & form_mask)   == form_mask)
+           || ((flg & misuse_mask) == misuse_mask) )
+            return;
+    }
+
+    /*
+     * Now fiddle the fOptSet bits, based on settings.
+     * The OPTPROC_LONGOPT bit is immutable, thus if it is set,
+     * then fnm points to a mask off mask.
+     */
     {
         ao_flag_names_t const * fnm = fn_table;
-
-        while (flg != 0) {
+        for (;;) {
             if ((flg & 1) != 0) {
                 if ((fnm->fnm_mask & OPTPROC_LONGOPT) != 0)
                      opts->fOptSet &= fnm->fnm_mask;
                 else opts->fOptSet |= fnm->fnm_mask;
             }
             flg >>= 1;
+            if (flg == 0)
+                break;
             fnm++;
         }
     }
@@ -228,28 +299,97 @@ optionOnlyUsage(tOptions * pOpts, int ex_code)
     prt_opt_usage(pOpts, ex_code, pOptTitle);
 
     fflush(option_usage_fp);
-    if (ferror(option_usage_fp) != 0) {
-        fputs(zOutputFail, stderr);
-        exit(EXIT_FAILURE);
-    }
+    if (ferror(option_usage_fp) != 0)
+        fserr_exit(pOpts->pzProgName, zwriting, (option_usage_fp == stderr)
+                   ? zstderr_name : zstdout_name);
 }
 
+LOCAL void
+ao_bug(char const * msg)
+{
+    fprintf(stderr, zao_bug_msg, msg);
+    exit(EX_SOFTWARE);
+}
+
+/**
+ * Print a message suggesting how to get help.
+ *
+ * @param[in] opts      the program options
+ */
+static void
+print_offer_usage(tOptions * opts)
+{
+    char help[24];
+
+    if (HAS_opt_usage_t(opts)) {
+        int ix = opts->presetOptCt;
+        tOptDesc * od = opts->pOptDesc + ix;
+        while (od->optUsage != AOUSE_HELP) {
+            if (++ix >= opts->optCt)
+                ao_bug(zmissing_help_msg);
+            od++;
+        }
+        switch (opts->fOptSet & (OPTPROC_LONGOPT | OPTPROC_SHORTOPT)) {
+        case OPTPROC_SHORTOPT:
+            help[0] = '-';
+            help[1] = od->optValue;
+            help[2] = NUL;
+            break;
+
+        case OPTPROC_LONGOPT:
+        case (OPTPROC_LONGOPT | OPTPROC_SHORTOPT):
+            help[0] = help[1] = '-';
+            strncpy(help + 2, od->pz_Name, 20);
+            break;
+        
+        case 0:
+            strncpy(help, od->pz_Name, 20);
+            break;
+        }
+
+    } else {
+        switch (opts->fOptSet & (OPTPROC_LONGOPT | OPTPROC_SHORTOPT)) {
+        case OPTPROC_SHORTOPT:
+            strcpy(help, "-h");
+            break;
+
+        case OPTPROC_LONGOPT:
+        case (OPTPROC_LONGOPT | OPTPROC_SHORTOPT):
+            strcpy(help, "--help");
+        
+        case 0:
+            strcpy(help, "help");
+            break;
+        }
+    }
+
+    fprintf(option_usage_fp, zoffer_usage_fmt, opts->pzProgName, help);
+}
+
+/**
+ * Print information about each option.
+ *
+ * @param[in] opts      the program options
+ * @param[in] exit_code whether or not there was a usage error reported.
+ *                      used to select full usage versus abbreviated.
+ */
 static void
 print_usage_details(tOptions * opts, int exit_code)
 {
     {
         char const * pOptTitle = NULL;
+        int flen;
 
         /*
          *  Determine which header and which option formatting strings to use
          */
         if (do_gnu_usage(opts)) {
-            int flen = setGnuOptFmts(opts, &pOptTitle);
+            flen = setGnuOptFmts(opts, &pOptTitle);
             sprintf(line_fmt_buf, zFmtFmt, flen);
             fputc(NL, option_usage_fp);
         }
         else {
-            int flen = setStdOptFmts(opts, &pOptTitle);
+            flen = setStdOptFmts(opts, &pOptTitle);
             sprintf(line_fmt_buf, zFmtFmt, flen);
 
             /*
@@ -263,6 +403,9 @@ print_usage_details(tOptions * opts, int exit_code)
                 fputs(pOptTitle, option_usage_fp);
         }
 
+        flen = 4 - ((flen + 15) / 8);
+        if (flen > 0)
+            tab_skip_ct = flen;
         prt_opt_usage(opts, exit_code, pOptTitle);
     }
 
@@ -303,18 +446,150 @@ print_usage_details(tOptions * opts, int exit_code)
 
     fflush(option_usage_fp);
 
-    if (ferror(option_usage_fp) != 0) {
-        fputs(zOutputFail, stderr);
-        exit(EXIT_FAILURE);
-    }
+    if (ferror(option_usage_fp) != 0)
+        fserr_exit(opts->pzProgName, zwriting, (option_usage_fp == stderr)
+                   ? zstderr_name : zstdout_name);
 }
 
+static void
+print_one_paragraph(char const * text, bool plain, FILE * fp)
+{
+    if (plain) {
+#ifdef ENABLE_NLS
+#ifdef HAVE_LIBINTL_H
+#ifdef DEBUG_ENABLED
+#undef gettext
+#endif
+        char * buf = dgettext("libopts", text);
+        if (buf == text)
+            text = gettext(text);
+#endif /* HAVE_LIBINTL_H */
+#endif /* ENABLE_NLS */
+        fputs(text, fp);
+    }
+
+    else {
+        char const * t = optionQuoteString(text, LINE_SPLICE);
+        fprintf(fp, PUTS_FMT, t);
+        AGFREE((void *)t);
+    }
+}
+ 
+/*=export_func  optionPrintParagraphs
+ * private:
+ *
+ * what:  Print a paragraph of usage text
+ * arg:   + char const * + text  + a block of text that has bee i18n-ed +
+ * arg:   + bool         + plain + false -> wrap text in fputs()        +
+ * arg:   + FILE *       + fp    + the stream file pointer for output   +
+ *
+ * doc:
+ *  This procedure is called in two contexts: when a full or short usage text
+ *  has been provided for display, and when autogen is assembling a list of
+ *  translatable texts in the optmain.tlib template.  In the former case, \a
+ *  plain is set to \a true, otherwise \a false.
+ *
+ *  Anything less than 256 characters in size is printed as a single unit.
+ *  Otherwise, paragraphs are detected.  A paragraph break is defined as just
+ *  before a non-empty line preceded by two newlines or a line that starts
+ *  with at least one space character but fewer than 8 space characters.
+ *  Lines indented with tabs or more than 7 spaces are considered continuation
+ *  lines.
+ *
+ *  If 'plain' is true, we are emitting text for a user to see.  So, if it is
+ *  true and NLS is not enabled, then just write the whole thing at once.
+=*/
+void
+optionPrintParagraphs(char const * text, bool plain, FILE * fp)
+{
+    size_t len = strlen(text);
+    char * buf;
+#ifndef ENABLE_NLS
+    if (plain || (len < 256))
+#else
+    if (len < 256)
+#endif
+    {
+        print_one_paragraph(text, plain, fp);
+        return;
+    }
+
+    AGDUPSTR(buf, text, "ppara");
+    text = buf;
+
+    for (;;) {
+        char * scan;
+
+        if (len < 256) {
+        done:
+            print_one_paragraph(buf, plain, fp);
+            break;
+        }
+        scan = buf;
+
+    try_longer:
+        scan = strchr(scan, NL);
+        if (scan == NULL)
+            goto done;
+
+        if ((scan - buf) < 40) {
+            scan++;
+            goto try_longer;
+        }
+
+        scan++;
+        if ((! isspace((int)*scan)) || (*scan == HT))
+            /*
+             * line starts with tab or non-whitespace --> continuation
+             */
+            goto try_longer;
+
+        if (*scan == NL) {
+            /*
+             * Double newline -> paragraph break
+             * Include all newlines in current paragraph.
+             */
+            while (*++scan == NL)  /*continue*/;
+
+        } else {
+            char * p = scan;
+            int   sp_ct = 0;
+
+            while (*p == ' ') {
+                if (++sp_ct >= 8) {
+                    /*
+                     * Too many spaces --> continuation line
+                     */
+                    scan = p;
+                    goto try_longer;
+                }
+                p++;
+            }
+        }
+
+        /*
+         * "scan" points to the first character of a paragraph or the
+         * terminating NUL byte.
+         */
+        {
+            char svch = *scan;
+            *scan = NUL;
+            print_one_paragraph(buf, plain, fp);
+            len -= scan - buf;
+            if (len <= 0)
+                break;
+            *scan = svch;
+            buf = scan;
+        }
+    }
+    AGFREE((void *)text);
+}
 
 /*=export_func  optionUsage
  * private:
  *
  * what:  Print usage text
- * arg:   + tOptions* + pOptions + program options descriptor +
+ * arg:   + tOptions* + opts + program options descriptor +
  * arg:   + int       + exitCode + exit code for calling exit(3) +
  *
  * doc:
@@ -327,12 +602,13 @@ print_usage_details(tOptions * opts, int exit_code)
  *  to stdout and the actual exit code will be "EXIT_SUCCESS".
 =*/
 void
-optionUsage(tOptions * pOptions, int usage_exit_code)
+optionUsage(tOptions * opts, int usage_exit_code)
 {
     int exit_code = (usage_exit_code == AO_EXIT_REQ_USAGE)
         ? EXIT_SUCCESS : usage_exit_code;
 
     displayEnum = false;
+    set_usage_flags(opts, NULL);
 
     /*
      *  Paged usage will preset option_usage_fp to an output file.
@@ -346,32 +622,42 @@ optionUsage(tOptions * pOptions, int usage_exit_code)
         char const * pz;
 
         if (exit_code == EXIT_SUCCESS) {
-            pz = (pOptions->structVersion >= 30 * 4096)
-                ? pOptions->pzFullUsage : NULL;
+            pz = (opts->structVersion >= 30 * 4096)
+                ? opts->pzFullUsage : NULL;
 
             if (option_usage_fp == NULL)
                 option_usage_fp = stdout;
         } else {
-            pz = (pOptions->structVersion >= 30 * 4096)
-                ? pOptions->pzShortUsage : NULL;
+            pz = (opts->structVersion >= 30 * 4096)
+                ? opts->pzShortUsage : NULL;
 
             if (option_usage_fp == NULL)
                 option_usage_fp = stderr;
         }
 
-        if (pz != NULL) {
-            fputs(pz, option_usage_fp);
-            exit(exit_code);
+        if (((opts->fOptSet & OPTPROC_COMPUTE) == 0) && (pz != NULL)) {
+            if ((opts->fOptSet & OPTPROC_TRANSLATE) != 0)
+                optionPrintParagraphs(pz, true, option_usage_fp);
+            else
+                fputs(pz, option_usage_fp);
+            goto flush_and_exit;
         }
     }
 
-    fprintf(option_usage_fp, pOptions->pzUsageTitle, pOptions->pzProgName);
-    set_usage_flags(pOptions, NULL);
+    fprintf(option_usage_fp, opts->pzUsageTitle, opts->pzProgName);
 
     if ((exit_code == EXIT_SUCCESS) ||
-        (! skip_misuse_usage(pOptions)))
+        (! skip_misuse_usage(opts)))
 
-        print_usage_details(pOptions, usage_exit_code);
+        print_usage_details(opts, usage_exit_code);
+    else
+        print_offer_usage(opts);
+    
+ flush_and_exit:
+    fflush(option_usage_fp);
+    if (ferror(option_usage_fp) != 0)
+        fserr_exit(opts->pzProgName, zwriting, (option_usage_fp == stdout)
+                   ? zstdout_name : zstderr_name);
 
     exit(exit_code);
 }
@@ -382,62 +668,74 @@ optionUsage(tOptions * pOptions, int usage_exit_code)
 /**
  * print option conflicts.
  *
- * @param pOptions the program option descriptor
- * @param pOD      the option descriptor
- * @param pAT      names of the option argument types
+ * @param opts the program option descriptor
+ * @param od   the option descriptor
  */
 static void
-prt_conflicts(tOptions * pOptions, tOptDesc * pOD)
+prt_conflicts(tOptions * opts, tOptDesc * od)
 {
-
-    fputs(zTabHyp, option_usage_fp);
+    const int * opt_no;
+    fputs(zTabHyp + tab_skip_ct, option_usage_fp);
 
     /*
      *  REQUIRED:
      */
-    if (pOD->pOptMust != NULL) {
-        const int* pOptNo = pOD->pOptMust;
+    if (od->pOptMust != NULL) {
+        opt_no = od->pOptMust;
 
-        fputs(zReqThese, option_usage_fp);
-        for (;;) {
-            fprintf(option_usage_fp, zTabout,
-                    pOptions->pOptDesc[*pOptNo].pz_Name);
-            if (*++pOptNo == NO_EQUIVALENT)
-                break;
+        if (opt_no[1] == NO_EQUIVALENT) {
+            fprintf(option_usage_fp, zReqOne,
+                    opts->pOptDesc[*opt_no].pz_Name);
+        } else {
+            fputs(zReqThese, option_usage_fp);
+            for (;;) {
+                fprintf(option_usage_fp, zTabout + tab_skip_ct,
+                        opts->pOptDesc[*opt_no].pz_Name);
+                if (*++opt_no == NO_EQUIVALENT)
+                    break;
+            }
         }
 
-        if (pOD->pOptCant != NULL)
-            fputs(zTabHypAnd, option_usage_fp);
+        if (od->pOptCant != NULL)
+            fputs(zTabHypAnd + tab_skip_ct, option_usage_fp);
     }
 
     /*
      *  CONFLICTS:
      */
-    if (pOD->pOptCant != NULL) {
-        const int* pOptNo = pOD->pOptCant;
+    if (od->pOptCant == NULL)
+        return;
 
-        fputs(zProhib, option_usage_fp);
-        for (;;) {
-            fprintf(option_usage_fp, zTabout,
-                    pOptions->pOptDesc[*pOptNo].pz_Name);
-            if (*++pOptNo == NO_EQUIVALENT)
-                break;
-        }
+    opt_no = od->pOptCant;
+
+    if (opt_no[1] == NO_EQUIVALENT) {
+        fprintf(option_usage_fp, zProhibOne,
+                opts->pOptDesc[*opt_no].pz_Name);
+        return;
+    }
+
+    fputs(zProhib, option_usage_fp);
+    for (;;) {
+        fprintf(option_usage_fp, zTabout + tab_skip_ct,
+                opts->pOptDesc[*opt_no].pz_Name);
+        if (*++opt_no == NO_EQUIVALENT)
+            break;
     }
 }
 
 /**
  *  Print the usage information for a single vendor option.
  *
- * @param pOpts     the program option descriptor
- * @param pOD       the option descriptor
- * @param pAT       names of the option argument types
+ * @param[in] opts    the program option descriptor
+ * @param[in] od      the option descriptor
+ * @param[in] argtp   names of the option argument types
+ * @param[in] usefmt  format for primary usage line
  */
 static void
-prt_one_vendor(tOptions * pOptions, tOptDesc * pOD,
-               arg_types_t * pAT, char const * usefmt)
+prt_one_vendor(tOptions *    opts,  tOptDesc *   od,
+               arg_types_t * argtp, char const * usefmt)
 {
-    prt_preamble(pOptions, pOD, pAT);
+    prt_preamble(opts, od, argtp);
 
     {
         char z[ 80 ];
@@ -448,52 +746,52 @@ prt_one_vendor(tOptions * pOptions, tOptDesc * pOD,
          *  when the option argument is required, base the type string on the
          *  argument type.
          */
-        if (pOD->fOptState & OPTST_ARG_OPTIONAL) {
-            pzArgType = pAT->pzOpt;
+        if (od->fOptState & OPTST_ARG_OPTIONAL) {
+            pzArgType = argtp->pzOpt;
 
-        } else switch (OPTST_GET_ARGTYPE(pOD->fOptState)) {
-        case OPARG_TYPE_NONE:        pzArgType = pAT->pzNo;   break;
-        case OPARG_TYPE_ENUMERATION: pzArgType = pAT->pzKey;  break;
-        case OPARG_TYPE_FILE:        pzArgType = pAT->pzFile; break;
-        case OPARG_TYPE_MEMBERSHIP:  pzArgType = pAT->pzKeyL; break;
-        case OPARG_TYPE_BOOLEAN:     pzArgType = pAT->pzBool; break;
-        case OPARG_TYPE_NUMERIC:     pzArgType = pAT->pzNum;  break;
-        case OPARG_TYPE_HIERARCHY:   pzArgType = pAT->pzNest; break;
-        case OPARG_TYPE_STRING:      pzArgType = pAT->pzStr;  break;
-        case OPARG_TYPE_TIME:        pzArgType = pAT->pzTime; break;
+        } else switch (OPTST_GET_ARGTYPE(od->fOptState)) {
+        case OPARG_TYPE_NONE:        pzArgType = argtp->pzNo;   break;
+        case OPARG_TYPE_ENUMERATION: pzArgType = argtp->pzKey;  break;
+        case OPARG_TYPE_FILE:        pzArgType = argtp->pzFile; break;
+        case OPARG_TYPE_MEMBERSHIP:  pzArgType = argtp->pzKeyL; break;
+        case OPARG_TYPE_BOOLEAN:     pzArgType = argtp->pzBool; break;
+        case OPARG_TYPE_NUMERIC:     pzArgType = argtp->pzNum;  break;
+        case OPARG_TYPE_HIERARCHY:   pzArgType = argtp->pzNest; break;
+        case OPARG_TYPE_STRING:      pzArgType = argtp->pzStr;  break;
+        case OPARG_TYPE_TIME:        pzArgType = argtp->pzTime; break;
         default:                     goto bogus_desc;
         }
 
         pzArgType = SPN_WHITESPACE_CHARS(pzArgType);
         if (*pzArgType == NUL)
-            snprintf(z, sizeof(z), "%s", pOD->pz_Name);
+            snprintf(z, sizeof(z), "%s", od->pz_Name);
         else
-            snprintf(z, sizeof(z), "%s=%s", pOD->pz_Name, pzArgType);
-        fprintf(option_usage_fp, usefmt, z, pOD->pzText);
+            snprintf(z, sizeof(z), "%s=%s", od->pz_Name, pzArgType);
+        fprintf(option_usage_fp, usefmt, z, od->pzText);
 
-        switch (OPTST_GET_ARGTYPE(pOD->fOptState)) {
+        switch (OPTST_GET_ARGTYPE(od->fOptState)) {
         case OPARG_TYPE_ENUMERATION:
         case OPARG_TYPE_MEMBERSHIP:
-            displayEnum = (pOD->pOptProc != NULL) ? true : displayEnum;
+            displayEnum = (od->pOptProc != NULL) ? true : displayEnum;
         }
     }
 
     return;
 
  bogus_desc:
-    fprintf(stderr, zInvalOptDesc, pOD->pz_Name);
-    exit(EX_SOFTWARE);
+    fprintf(stderr, zbad_od, opts->pzProgName, od->pz_Name);
+    ao_bug(zbad_arg_type_msg);
 }
 
 /**
  * Print the long options processed with "-W".  These options will be the
  * ones that do *not* have flag characters.
  *
- * @param pOptions the program option descriptor
- * @param pOD      the option descriptor
+ * @param opts  the program option descriptor
+ * @param title the title for the options
  */
 static void
-prt_vendor_opts(tOptions * pOpts, char const * pOptTitle)
+prt_vendor_opts(tOptions * opts, char const * title)
 {
     static unsigned int const not_vended_mask =
         OPTST_NO_USAGE_MASK | OPTST_DOCUMENT;
@@ -505,51 +803,57 @@ prt_vendor_opts(tOptions * pOpts, char const * pOptTitle)
      *  Only handle client specified options.  The "vendor option" follows
      *  "presetOptCt", so we won't loop/recurse indefinitely.
      */
-    int          ct     = pOpts->presetOptCt;
-    tOptDesc *   pOD    = pOpts->pOptDesc;
-    size_t       nmlen  = 0;
+    int          ct     = opts->presetOptCt;
+    tOptDesc *   od     = opts->pOptDesc;
+    fprintf(option_usage_fp, zTabout + tab_skip_ct, zVendOptsAre);
 
-    fprintf(option_usage_fp, zTabout, zVendOptsAre);
+    {
+        size_t   nmlen  = 0;
+        do  {
+            size_t l;
+            if (  ((od->fOptState & not_vended_mask) != 0)
+               || IS_GRAPHIC_CHAR(od->optValue))
+                continue;
+
+            l = strlen(od->pz_Name);
+            if (l > nmlen)  nmlen = l;
+        } while (od++, (--ct > 0));
+
+        snprintf(vfmt, sizeof(vfmt), vfmtfmt, (unsigned int)nmlen + 4);
+    }
+
+    if (tab_skip_ct > 0)
+        tab_skip_ct--;
+
+    ct    = opts->presetOptCt;
+    od    = opts->pOptDesc;
 
     do  {
-        size_t l;
-        if (  ((pOD->fOptState & not_vended_mask) != 0)
-           || IS_GRAPHIC_CHAR(pOD->optValue))
+        if (  ((od->fOptState & not_vended_mask) != 0)
+           || IS_GRAPHIC_CHAR(od->optValue))
             continue;
 
-        l = strlen(pOD->pz_Name);
-        if (l > nmlen)  nmlen = l;
-    } while (pOD++, (--ct > 0));
+        prt_one_vendor(opts, od, &argTypes, vfmt);
+        prt_extd_usage(opts, od, title);
 
-    sprintf(vfmt, vfmtfmt, (unsigned int)nmlen + 4);
-    ct     = pOpts->presetOptCt;
-    pOD    = pOpts->pOptDesc;
+    } while (od++, (--ct > 0));
 
-    do  {
-        if (  ((pOD->fOptState & not_vended_mask) != 0)
-           || IS_GRAPHIC_CHAR(pOD->optValue))
-            continue;
-
-        prt_one_vendor(pOpts, pOD, &argTypes, vfmt);
-        prt_extd_usage(pOpts, pOD, pOptTitle);
-
-    } while (pOD++, (--ct > 0));
+    /* no need to restore "tab_skip_ct" - options are done now */
 }
 
 /**
  * Print extended usage.  Usage/help was requested.
  *
- * @param pOptions the program option descriptor
- * @param pOD      the option descriptor
- * @param pAT      names of the option argument types
+ * @param opts  the program option descriptor
+ * @param od   the option descriptor
+ * @param title the title for the options
  */
 static void
-prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
-               char const * pOptTitle)
+prt_extd_usage(tOptions * opts, tOptDesc * od, char const * title)
 {
-    if (  ((pOpts->fOptSet & OPTPROC_VENDOR_OPT) != 0)
-       && (pOD->optActualValue == VENDOR_OPTION_VALUE)) {
-        prt_vendor_opts(pOpts, pOptTitle);
+    if (  ((opts->fOptSet & OPTPROC_VENDOR_OPT) != 0)
+       && (od->optActualValue == VENDOR_OPTION_VALUE)) {
+        prt_vendor_opts(opts, title);
         return;
     }
 
@@ -557,34 +861,33 @@ prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
      *  IF there are option conflicts or dependencies,
      *  THEN print them here.
      */
-    if (  (pOD->pOptMust != NULL)
-       || (pOD->pOptCant != NULL) )
-        prt_conflicts(pOpts, pOD);
+    if ((od->pOptMust != NULL) || (od->pOptCant != NULL))
+        prt_conflicts(opts, od);
 
     /*
      *  IF there is a disablement string
      *  THEN print the disablement info
      */
-    if (pOD->pz_DisableName != NULL )
-        fprintf(option_usage_fp, zDis, pOD->pz_DisableName);
+    if (od->pz_DisableName != NULL )
+        fprintf(option_usage_fp, zDis + tab_skip_ct, od->pz_DisableName);
 
     /*
      *  Check for argument types that have callbacks with magical properties
      */
-    switch (OPTST_GET_ARGTYPE(pOD->fOptState)) {
+    switch (OPTST_GET_ARGTYPE(od->fOptState)) {
     case OPARG_TYPE_NUMERIC:
         /*
          *  IF the numeric option has a special callback,
          *  THEN call it, requesting the range or other special info
          */
-        if (  (pOD->pOptProc != NULL)
-           && (pOD->pOptProc != optionNumericVal) ) {
-            (*(pOD->pOptProc))(OPTPROC_EMIT_USAGE, pOD);
+        if (  (od->pOptProc != NULL)
+           && (od->pOptProc != optionNumericVal) ) {
+            (*(od->pOptProc))(OPTPROC_EMIT_USAGE, od);
         }
         break;
 
     case OPARG_TYPE_FILE:
-        (*(pOD->pOptProc))(OPTPROC_EMIT_USAGE, pOD);
+        (*(od->pOptProc))(OPTPROC_EMIT_USAGE, od);
         break;
     }
 
@@ -592,18 +895,18 @@ prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
      *  IF the option defaults to being enabled,
      *  THEN print that out
      */
-    if (pOD->fOptState & OPTST_INITENABLED)
-        fputs(zEnab, option_usage_fp);
+    if (od->fOptState & OPTST_INITENABLED)
+        fputs(zEnab + tab_skip_ct, option_usage_fp);
 
     /*
      *  IF  the option is in an equivalence class
      *        AND not the designated lead
      *  THEN print equivalence and leave it at that.
      */
-    if (  (pOD->optEquivIndex != NO_EQUIVALENT)
-       && (pOD->optEquivIndex != pOD->optActualIndex )  )  {
-        fprintf(option_usage_fp, zAlt,
-                 pOpts->pOptDesc[ pOD->optEquivIndex ].pz_Name);
+    if (  (od->optEquivIndex != NO_EQUIVALENT)
+       && (od->optEquivIndex != od->optActualIndex )  )  {
+        fprintf(option_usage_fp, zalt_opt + tab_skip_ct,
+                 opts->pOptDesc[ od->optEquivIndex ].pz_Name);
         return;
     }
 
@@ -613,32 +916,33 @@ prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
      *    AND it is not an auto-managed option (e.g. --help, et al.)
      *  THEN advise that this option may not be preset.
      */
-    if (  ((pOD->fOptState & OPTST_NO_INIT) != 0)
-       && (  (pOpts->papzHomeList != NULL)
-          || (pOpts->pzPROGNAME != NULL)
+    if (  ((od->fOptState & OPTST_NO_INIT) != 0)
+       && (  (opts->papzHomeList != NULL)
+          || (opts->pzPROGNAME != NULL)
           )
-       && (pOD->optIndex < pOpts->presetOptCt)
+       && (od->optIndex < opts->presetOptCt)
        )
 
-        fputs(zNoPreset, option_usage_fp);
+        fputs(zNoPreset + tab_skip_ct, option_usage_fp);
 
     /*
      *  Print the appearance requirements.
      */
-    if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_MEMBERSHIP)
-        fputs(zMembers, option_usage_fp);
+    if (OPTST_GET_ARGTYPE(od->fOptState) == OPARG_TYPE_MEMBERSHIP)
+        fputs(zMembers + tab_skip_ct, option_usage_fp);
 
-    else switch (pOD->optMinCt) {
+    else switch (od->optMinCt) {
     case 1:
     case 0:
-        switch (pOD->optMaxCt) {
-        case 0:       fputs(zPreset, option_usage_fp); break;
-        case NOLIMIT: fputs(zNoLim, option_usage_fp);  break;
+        switch (od->optMaxCt) {
+        case 0:       fputs(zPreset + tab_skip_ct, option_usage_fp); break;
+        case NOLIMIT: fputs(zNoLim  + tab_skip_ct, option_usage_fp); break;
         case 1:       break;
             /*
              * IF the max is more than one but limited, print "UP TO" message
              */
-        default:      fprintf(option_usage_fp, zUpTo, pOD->optMaxCt);  break;
+        default:
+            fprintf(option_usage_fp, zUpTo + tab_skip_ct, od->optMaxCt); break;
         }
         break;
 
@@ -646,12 +950,13 @@ prt_extd_usage(tOptions * pOpts, tOptDesc * pOD,
         /*
          *  More than one is required.  Print the range.
          */
-        fprintf(option_usage_fp, zMust, pOD->optMinCt, pOD->optMaxCt);
+        fprintf(option_usage_fp, zMust + tab_skip_ct,
+                od->optMinCt, od->optMaxCt);
     }
 
-    if (  NAMED_OPTS(pOpts)
-       && (pOpts->specOptIdx.default_opt == pOD->optIndex))
-        fputs(zDefaultOpt, option_usage_fp);
+    if (  NAMED_OPTS(opts)
+       && (opts->specOptIdx.default_opt == od->optIndex))
+        fputs(zDefaultOpt + tab_skip_ct, option_usage_fp);
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -802,7 +1107,7 @@ prt_one_usage(tOptions * opts, tOptDesc * od, arg_types_t * at)
     return;
 
  bogus_desc:
-    fprintf(stderr, zInvalOptDesc, od->pz_Name);
+    fprintf(stderr, zbad_od, opts->pzProgName, od->pz_Name);
     exit(EX_SOFTWARE);
 }
 
@@ -1048,8 +1353,8 @@ setStdOptFmts(tOptions * opts, char const ** ptxt)
     return flen;
 }
 
-
-/*:
+/** @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
