@@ -2,7 +2,7 @@
 static const char cright_years_z[] =
 
 /* uuencode utility.
-   Copyright (C) */ "1994-1996, 2002, 2005-2007, 2010-2011";
+   Copyright (C) */ "1994-2013";
 
 /* Free Software Foundation, Inc.
 
@@ -59,45 +59,27 @@ static const char cright_years_z[] =
 | Encode a file so it can be mailed to a remote system.	 |
 \=======================================================*/
 
-#include "local.h"
+#define  UUENCODE_C  1
+#include "uuencode-opts.h"
 
 #if HAVE_LOCALE_H
 #else
 # define setlocale(Category, Locale)
 #endif
-#define _(str) gettext (str)
 
 #if __CYGWIN__
 # include <fcntl.h>
 # include <io.h>
 #endif
 
-#define	RW (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)
+#define	IRWALL_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)
+char * output_name = NULL;
+char * input_name = NULL;
 
-static struct option longopts[] =
-{
-  { "base64", 0, 0, 'm' },
-  { "version", 0, 0, 'v' },
-  { "help", 0, 0, 'h' },
-  { NULL, 0, 0, 0 }
-};
+static inline void try_putchar (int);
+static void encode (void);
 
-static inline void try_putchar __P ((int));
-static void encode __P ((void));
-static void usage __P ((int))
-#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-     __attribute__ ((noreturn))
-#endif
-;
-
-/* The name this program was run with. */
-const char *program_name;
-
-/* Pointer to the translation table we currently use.  */
-const char *trans_ptr;
-
-/* The two currently defined translation tables.  The first is the
-   standard uuencoding, the second is base64 encoding.  */
+/* standard uuencoding translation table  */
 const char uu_std[64] =
 {
   '`', '!', '"', '#', '$', '%', '&', '\'',
@@ -110,84 +92,100 @@ const char uu_std[64] =
   'X', 'Y', 'Z', '[', '\\', ']', '^', '_'
 };
 
-const char uu_base64[64] =
-{
-  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
-  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
-  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
-  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
-  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
-  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
-  'w', 'x', 'y', 'z', '0', '1', '2', '3',
-  '4', '5', '6', '7', '8', '9', '+', '/'
-};
+/* Pointer to the translation table we currently use.  */
+const char * trans_ptr = uu_std;
 
 /* ENC is the basic 1 character encoding function to make a char printing.  */
 #define ENC(Char) (trans_ptr[(Char) & 077])
 
 static inline void
-try_putchar (c)
-         int c;
+try_putchar (int c)
 {
   if (putchar (c) == EOF)
-    error (EXIT_FAILURE, 0, _("Write error"));
+    fserr (UUENCODE_EXIT_FAILURE, "putchar", _("standard output"));
 }
 
 /*------------------------------------------------.
 | Copy from IN to OUT, encoding as you go along.  |
 `------------------------------------------------*/
 
-static void
-encode ()
+static size_t
+encode_block (char * out, unsigned char const * in, size_t in_len)
 {
-  register int n;
-  int finishing = 0;
-  register char *p;
-  char buf[46];  /* 45 should be enough, but one never knows... */
+  char * start = out;
 
-  while ( !finishing && (n = fread (buf, 1, 45, stdin)) > 0 )
+  while (in_len >= 3)
     {
-      if (n < 45)
-        {
-          if (feof (stdin))
-            finishing = 1;
-          else
-            error (EXIT_FAILURE, 0, _("Read error"));
-        }
-
-      if (trans_ptr == uu_std)
-        putchar (ENC (n));
-
-      for (p = buf; n > 2; n -= 3, p += 3)
-        {
-          try_putchar (ENC (*p >> 2));
-          try_putchar (ENC (((*p << 4) & 060) | ((p[1] >> 4) & 017)));
-          try_putchar (ENC (((p[1] << 2) & 074) | ((p[2] >> 6) & 03)));
-          try_putchar (ENC (p[2] & 077));
-        }
-
-      if (n > 0)  /* encode the last one or two chars */
-        {
-          char tail = trans_ptr == uu_std ? ENC ('\0') : '=';
-
-          if (n == 1)
-            p[1] = '\0';
-
-          try_putchar (ENC (*p >> 2));
-          try_putchar (ENC (((*p << 4) & 060) | ((p[1] >> 4) & 017)));
-          try_putchar (n == 1 ? tail : ENC ((p[1] << 2) & 074));
-          try_putchar (tail);
-        }
-
-      try_putchar ('\n');
+      *(out++) = ENC (in[0] >> 2);
+      *(out++) = ENC (((in[0] & 0x03) << 4) + ((in[1] >> 4) & 0x0F));
+      *(out++) = ENC (((in[1] & 0x0F) << 2) + ((in[2] >> 6) & 0x03));
+      *(out++) = ENC (in[2] & 0x3F);
+      in_len -= 3;
+      in += 3;
     }
 
-  if (ferror (stdin))
-    error (EXIT_FAILURE, 0, _("Read error"));
-  if (fclose (stdin) != 0)
-    error (EXIT_FAILURE, errno, _("Read error"));
+  if (in_len > 0)
+    {
+      unsigned char lc = (in_len > 1) ? in[1] : '\0';
+      *(out++) = ENC (in[0] >> 2);
+      *(out++) = ENC (((in[0] & 0x03) << 4) + ((lc >> 4) & 0x0F));
+      *(out++) = ENC ((lc & 0x0F) << 2);
+      *(out++) = ENC (0);
+    }
 
-  if (trans_ptr == uu_std)
+  return out - start;
+}
+
+/*------------------------------------------.
+| Copy from IN to OUT, encoding as you go.  |
+`------------------------------------------*/
+
+static void
+encode (void)
+{
+  int finishing = 0;
+
+  do
+    {
+      unsigned char buf[45];
+      char   buf_out[64];
+      int    rdct = fread (buf, 1, sizeof(buf), stdin);
+      size_t wrct = sizeof (buf_out);
+
+      if (rdct <= 0)
+        {
+          if (ferror (stdin))
+            fserr (UUENCODE_EXIT_FAILURE, "fread", input_name);
+          break;
+        }
+
+      if (rdct < 45)
+        {
+          if (! feof (stdin))
+            fserr (UUENCODE_EXIT_FAILURE, "fread", input_name);
+          finishing = 1;
+        }
+
+      if (! HAVE_OPT(BASE64))
+        {
+          try_putchar (ENC ((unsigned int)rdct));
+          wrct = encode_block (buf_out, buf, rdct);
+        }
+      else
+        {
+          base64_encode ((char *)buf, (size_t)rdct, buf_out, wrct);
+          wrct = strlen(buf_out);
+        }
+
+      buf_out[wrct++] = '\n';
+      if (fwrite (buf_out, 1, wrct, stdout) != wrct)
+        fserr (UUENCODE_EXIT_FAILURE, "fwrite", _("standard output"));
+    } while (! finishing);
+
+  if (fclose (stdin) != 0)
+    fserr (UUENCODE_EXIT_FAILURE, "fclose", input_name);
+
+  if (! HAVE_OPT(BASE64))
     {
       try_putchar (ENC ('\0'));
       try_putchar ('\n');
@@ -195,90 +193,41 @@ encode ()
 }
 
 static void
-usage (status)
-     int status;
+process_opts (int argc, char ** argv, int * mode)
 {
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("Usage: %s [INFILE] REMOTEFILE\n"), program_name);
-      fputs (_("\n\
-  -m, --base64    use base64 encoding as of RFC1521\n\
-      --help      display this help and exit\n\
-      --version   output version information and exit\n"), stdout);
-      /* TRANSLATORS: add the contact address for your translation team! */
-      printf (_("Report bugs to <%s>.\n"), PACKAGE_BUGREPORT);
-    }
-  exit (status);
-}
-
-int
-main (argc, argv)
-     int argc;
-     char *const *argv;
-{
-  int opt;
-  struct stat sb;
-  int mode;
-
   /* Set global variables.  */
-  trans_ptr = uu_std;		/* Standard encoding is old uu format.  */
-
-  program_name = argv[0];
   setlocale (LC_ALL, "");
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while (opt = getopt_long (argc, argv, "hm", longopts, (int *) NULL),
-	 opt != EOF)
-    {
-      switch (opt)
-	{
-	case 'h':
-	  usage (EXIT_SUCCESS);
+  input_name = _("standard input");
 
-	case 'm':
-	  trans_ptr = uu_base64;
-	  break;
+  {
+    int ct = optionProcess (&uuencodeOptions, argc, argv);
+    argc -= ct;
+    argv += ct;
+  }
 
-	case 'v':
-	  printf ("%s (GNU %s) %s\n", basename (program_name),
-		  PACKAGE, VERSION);
-	  /* xgettext: no-wrap */
-	  printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
-		  cright_years_z);
-	  exit (EXIT_SUCCESS);
-
-	case 0:
-	  break;
-
-	default:
-	  usage (EXIT_FAILURE);
-	}
-    }
-
-  switch (argc - optind)
+  switch (argc)
     {
     case 2:
       /* Optional first argument is input file.  */
       {
+        struct stat sb;
 #if S_IRWXU != 0700
         choke me - Must translate mode argument
 #endif
 
-	FILE *fp = freopen (argv[optind], FOPEN_READ_BINARY, stdin);
+	FILE * fp = freopen (*argv, FOPEN_READ_BINARY, stdin);
+        input_name = *argv;
 	if (fp != stdin)
-	  error (EXIT_FAILURE, errno, _("fopen-ing %s"), argv[optind]);
+          fserr (UUENCODE_EXIT_FAILURE, _("freopen of stdin"), input_name);
 	if (fstat (fileno (stdin), &sb) != 0)
-	  error (EXIT_FAILURE, errno, _("fstat-ing %s"), argv[optind]);
-	mode = sb.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO);
-	optind++;
+          fserr (UUENCODE_EXIT_FAILURE, "fstat", input_name);
+	*mode = sb.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO);
+        output_name = argv[1];
 	break;
       }
 
@@ -288,24 +237,54 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
        setmode (STDIN_FILENO, O_BINARY);
 #endif
 
-      mode = RW & ~umask (RW);
+      *mode = IRWALL_MODE & ~umask (IRWALL_MODE);
+      output_name = *argv;
       break;
 
     case 0:
     default:
-      usage (EXIT_FAILURE);
+      USAGE (UUENCODE_EXIT_USAGE_ERROR);
     }
 
-  if (printf ("begin%s %o %s\n", trans_ptr == uu_std ? "" : "-base64",
-	      mode, argv[optind]) < 0)
-    error (EXIT_FAILURE, errno, _("Write error"));
+  if (HAVE_OPT(ENCODE_FILE_NAME))
+    {
+      size_t nmlen = strlen (output_name);
+      size_t bfsz  = nmlen + (nmlen / 3) + 4;;
+      char * p = malloc (bfsz);
+      if (p == NULL)
+        fserr (UUENCODE_EXIT_FAILURE, "malloc", _("file name"));
+      base64_encode ((char *)output_name, nmlen, p, bfsz);
+      output_name = p;
+    }
+}
+
+int
+main (int argc, char ** argv)
+{
+  int mode;
+  process_opts (argc, argv, &mode);
+
+  if (printf ("begin%s%s %o %s\n",
+              HAVE_OPT(BASE64) ? "-base64" : "",
+              HAVE_OPT(ENCODE_FILE_NAME) ? "-encoded" : "",
+	      mode, output_name) < 0)
+    fserr (UUENCODE_EXIT_FAILURE, "printf", _("standard output"));
 
   encode ();
 
-  if (ferror (stdout) ||
-      printf (trans_ptr == uu_std ? "end\n" : "====\n") < 0 ||
-      fclose (stdout) != 0)
-    error (EXIT_FAILURE, errno, _("Write error"));
+  if (ferror (stdout))
+    fserr (UUENCODE_EXIT_FAILURE, "ferror", _("standard output"));
+  if (puts (HAVE_OPT(BASE64) ? "====" : "end") == EOF)
+    fserr (UUENCODE_EXIT_FAILURE, "puts", _("standard output"));
+  if (fclose (stdout) != 0)
+    fserr (UUENCODE_EXIT_FAILURE, "fclose", _("standard output"));
 
-  exit (EXIT_SUCCESS);
+  exit (UUENCODE_EXIT_SUCCESS);
 }
+/*
+ * Local Variables:
+ * mode: C
+ * c-file-style: "gnu"
+ * indent-tabs-mode: nil
+ * End:
+ * end of src/uuencode.c */
